import { AnyIterable } from 'streaming-iterables';
import { Multiaddr } from '@multiformats/multiaddr';
import * as _libp2p_interface_peer_id from '@libp2p/interface-peer-id';
import { PeerId } from '@libp2p/interface-peer-id';
import * as _libp2p_interface_connection_dist_src from '@libp2p/interface-connection/dist/src';
import * as _multiformats_multiaddr_dist_src from '@multiformats/multiaddr/dist/src';
import { IncomingStreamData } from '@libp2p/interface-registrar';
import { Channel } from 'queueable';
import { Libp2p } from 'libp2p';

declare const newChannel: <T, S>(c: Channel<T>, i: IncomingStreamData, ctrl?: Channel<ControlMsg>, context?: S) => {
    send: (value: T) => Promise<void>;
    done: (err?: Error) => Promise<void>;
    id: StreamID;
    protocol: string;
    context: S;
    stat: {
        remoteAddr: _multiformats_multiaddr_dist_src.Multiaddr;
        remotePeer: _libp2p_interface_peer_id.PeerId;
        encryption: string;
        multiplexer: string;
        direction: _libp2p_interface_connection_dist_src.Direction;
        open: () => boolean;
    };
};

type Chan<T, S> = ReturnType<typeof newChannel<T, S>>;
type IterableFunc<I, O> = (data: AnyIterable<I>, incomingData: IncomingStreamData) => AnyIterable<O> | Promise<AnyIterable<O>>;
type PeerAddr = string | Multiaddr | PeerId;
interface FuncParams<I, O, S> extends Chan<O, S> {
    data: I;
}
interface MiddlewareParams<I, O, S extends {}> extends FuncParams<I, O, S> {
    next: (params?: Partial<FuncParams<I, O, S>>) => Promise<void>;
}
type Func<I, O, S extends {} = {}> = (params: FuncParams<I, O, S>) => Promise<void> | void;
type Service<I, O, S extends {} = {}> = (chan: Chan<O, S>) => Promise<Func<I, O, S>> | Func<I, O, S> | void | Promise<void>;
interface Codec<T> {
    encoder: (data: T) => Uint8Array | Promise<Uint8Array>;
    decoder: (data: Uint8Array) => T | Promise<T>;
}
interface InitOptions<T, S extends {}> {
    codec?: Codec<T>;
    context?: S;
}
interface ServerInitOptions<I extends T, O extends T, T, S extends {}> extends InitOptions<T, S> {
    middleware?: (params: MiddlewareParams<I, O, S>) => void | Promise<void>;
}
interface TransportChannel<I, O, S> extends AsyncIterableIterator<I>, Chan<O, S> {
}
interface MetaPointError extends Error {
    type: "error";
    id: StreamID;
}
interface MetaPointSuccess {
    type: "success";
    id: StreamID;
}
type ControlMsg = MetaPointError | MetaPointSuccess;
interface StreamID {
    connection: string;
    stream: string;
}

declare const client: (node: Libp2p, peer: PeerAddr | PeerAddr[]) => Promise<(<I extends T, O extends T, T = any, Context extends {} = {}>(name: string, options?: InitOptions<T, Context>) => Promise<((value: I) => Promise<O[]>) & TransportChannel<O, I, Context>>) & {
    close: () => Promise<void>;
}>;

declare const server: (node: Libp2p) => Promise<{
    handle: <I extends T, O extends T, T = any, Context extends {} = {}>(name: string, func: Func<I, O, Context>, options?: ServerInitOptions<I, O, T, Context>) => Promise<void>;
    serve: <I_1 extends T_1, O_1 extends T_1, T_1 = any, Context_1 extends {} = {}>(name: string, func: Service<I_1, O_1, Context_1>, options?: ServerInitOptions<I_1, O_1, T_1, Context_1>) => Promise<void>;
}>;

export { Chan, Codec, ControlMsg, Func, FuncParams, InitOptions, IterableFunc, MetaPointError, MetaPointSuccess, MiddlewareParams, PeerAddr, ServerInitOptions, Service, StreamID, TransportChannel, client, server };
